import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import {
  NewsletterViewModel,
  CreateNewsletterViewModel,
  SyncNewsletterViewModel,
  NewsletterConfigViewModel,
  NewsletterDatesViewModel,
  NewsletterRecipientsViewModel,
  SetNewsletterConfigViewModel,
  SyncNewsletterDatesViewModel,
  SyncNewsletterRecipientsViewModel,
} from "../../viewmodels/admin/newsletter.models";
import { BaseClient } from "../../clients/clientBase";

interface INewsletterRequests {
  getNewsletters: RequestDefinition<void, void, NewsletterViewModel[]>;
  getNewsletterById: RequestDefinition<{ id: number }, void, NewsletterViewModel>;
  getNewsletterDatesByNewsletter: RequestDefinition<{ newsletterId: number }, void, NewsletterDatesViewModel>;
  getNewsletterRecipientsByNewsletter: RequestDefinition<{ newsletterId: number }, void, NewsletterRecipientsViewModel>;
  getNewsletterPrintoutsByNewsletter: RequestDefinition<{ newsletterId: number }, void, Array<string>>;
  getNewsletterPrintoutsByNewsletterAndId: RequestDefinition<{ newsletterId: number; filename: number }, void, File>;
  createNewsletter: RequestDefinition<void, CreateNewsletterViewModel, number>;
  syncNewsletter: RequestDefinition<{ id: number }, SyncNewsletterViewModel, void>;
  syncNewsletterDatesByNewsletter: RequestDefinition<{ newsletterId: number }, SyncNewsletterDatesViewModel, void>;
  syncNewsletterRecipientsByNewsletter: RequestDefinition<{ newsletterId: number }, SyncNewsletterRecipientsViewModel, void>;

  getNewsletterConfigs: RequestDefinition<void, void, NewsletterConfigViewModel[]>;
  getNewsletterConfigById: RequestDefinition<{ communicationTypeId: number }, void, NewsletterConfigViewModel>;
  updateNewsletterConfig: RequestDefinition<void, SetNewsletterConfigViewModel, NewsletterViewModel>;
  resetNewsletterConfig: RequestDefinition<{ communicationTypeId: number }, void, NewsletterViewModel>;
}

@ImplementsRequestInterface<INewsletterRequests>()
export default abstract class NewsletterRequests {
  static async getNewsletters({ http }: BaseClient): Promise<AxiosResponse<NewsletterViewModel[], any>> {
    return await http.get("/admin/newsletter");
  }
  static async getNewsletterById({ http }: BaseClient, p: RequestData<{ id: number }, void>): Promise<AxiosResponse<NewsletterViewModel, any>> {
    return await http.get(`/admin/newsletter/${p.params.id}`);
  }
  static async getNewsletterDatesByNewsletter(
    { http }: BaseClient,
    p: RequestData<{ newsletterId: number }, void>
  ): Promise<AxiosResponse<NewsletterDatesViewModel, any>> {
    return await http.get(`/admin/newsletter/${p.params.newsletterId}/dates`);
  }
  static async getNewsletterRecipientsByNewsletter(
    { http }: BaseClient,
    p: RequestData<{ newsletterId: number }, void>
  ): Promise<AxiosResponse<NewsletterRecipientsViewModel, any>> {
    return await http.get(`/admin/newsletter/${p.params.newsletterId}/recipients`);
  }
  static async getNewsletterPrintoutsByNewsletter(
    { http }: BaseClient,
    p: RequestData<{ newsletterId: number }, void>
  ): Promise<AxiosResponse<Array<string>, any>> {
    return await http.get(`/admin/newsletter/${p.params.newsletterId}/printouts`);
  }
  static async getNewsletterPrintoutsByNewsletterAndId(
    { http }: BaseClient,
    p: RequestData<{ newsletterId: number; filename: number }, void>
  ): Promise<AxiosResponse<File, any>> {
    return await http.get(`/admin/newsletter/${p.params.newsletterId}/printouts/${p.params.filename}`);
  }
  static async createNewsletter({ http }: BaseClient, p: RequestData<void, CreateNewsletterViewModel>): Promise<AxiosResponse<number, any>> {
    return await http.post("/admin/newsletter", p.body);
  }
  static async updateNewsletter({ http }: BaseClient, p: RequestData<{ id: number }, SyncNewsletterViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/newsletter/${p.params.id}`, p.body);
  }
  static async syncNewsletter({ http }: BaseClient, p: RequestData<{ id: number }, SyncNewsletterViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/newsletter/${p.params.id}/synchronize`, p.body);
  }
  static async syncNewsletterDatesByNewsletter(
    { http }: BaseClient,
    p: RequestData<{ newsletterId: number }, SyncNewsletterDatesViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/newsletter/${p.params.newsletterId}/synchronize/dates`, p.body);
  }
  static async syncNewsletterRecipientsByNewsletter(
    { http }: BaseClient,
    p: RequestData<{ newsletterId: number }, SyncNewsletterRecipientsViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/newsletter/${p.params.newsletterId}/synchronize/recipients`, p.body);
  }

  static async getNewsletterConfigs({ http }: BaseClient): Promise<AxiosResponse<NewsletterConfigViewModel[], any>> {
    return await http.get("/admin/newsletterconfig");
  }
  static async getNewsletterConfigById(
    { http }: BaseClient,
    p: RequestData<{ communicationTypeId: number }, void>
  ): Promise<AxiosResponse<NewsletterConfigViewModel, any>> {
    return await http.get(`/admin/newsletterconfig/${p.params.communicationTypeId}`);
  }
  static async updateNewsletterConfig(
    { http }: BaseClient,
    p: RequestData<void, SetNewsletterConfigViewModel>
  ): Promise<AxiosResponse<NewsletterViewModel, any>> {
    return await http.put("/admin/newsletterconfig", p.body);
  }
  static async resetNewsletterConfig(
    { http }: BaseClient,
    p: RequestData<{ communicationTypeId: number }, void>
  ): Promise<AxiosResponse<NewsletterViewModel, any>> {
    return await http.delete(`/admin/newsletterconfig/${p.params.communicationTypeId}`);
  }
}
