import { AxiosResponse } from "axios";
import { RequestDefinition, RequestData, ImplementsRequestInterface } from "../requestBase";
import {
  UpdateUserViewModel,
  UserViewModel,
  CreateInviteViewModel,
  InviteViewModel,
  UpdateUserPermissionViewModel,
  UpdateUserRolesViewModel,
} from "../../viewmodels/admin/user.models";
import { PermissionObject, PermissionString } from "../../types/admin/permissionTypes";
import { RoleViewModel } from "../../viewmodels/admin/role.models";
import { BaseClient } from "../../clients/clientBase";

interface IUserRequests {
  getUsers: RequestDefinition<void, void, UserViewModel[]>;
  getUserById: RequestDefinition<{ id: string }, void, UserViewModel>;
  getUserPermissionsById: RequestDefinition<{ userId: string }, void, PermissionObject>;
  getUserRolesById: RequestDefinition<{ userId: string }, void, RoleViewModel[]>;
  updateUser: RequestDefinition<{ id: string }, UpdateUserViewModel, void>;
  updateUserPermissions: RequestDefinition<{ userId: string }, UpdateUserPermissionViewModel, void>;
  updateUserRoles: RequestDefinition<{ userId: string }, UpdateUserRolesViewModel, void>;
  deleteUser: RequestDefinition<{ id: string }, void, void>;

  getInvites: RequestDefinition<void, void, InviteViewModel[]>;
  createInvite: RequestDefinition<void, CreateInviteViewModel, void>;
  deleteInvite: RequestDefinition<{ mail: string }, void, void>;
}

@ImplementsRequestInterface<IUserRequests>()
export default abstract class UserRequests {
  static async getUsers({ http }: BaseClient): Promise<AxiosResponse<UserViewModel[], any>> {
    return await http.get("/admin/user");
  }
  static async getUserById({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<UserViewModel, any>> {
    return await http.get(`/admin/user/${p.params.id}`);
  }
  static async getUserPermissionsById({ http }: BaseClient, p: RequestData<{ userId: string }, void>): Promise<AxiosResponse<PermissionObject, any>> {
    return await http.get(`/admin/user/${p.params.userId}/permissions`);
  }
  static async getUserRolesById({ http }: BaseClient, p: RequestData<{ userId: string }, void>): Promise<AxiosResponse<RoleViewModel[], any>> {
    return await http.get(`/admin/user/${p.params.userId}/roles`);
  }
  static async updateUser({ http }: BaseClient, p: RequestData<{ id: string }, UpdateUserViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/user/${p.params.id}`, p.body);
  }
  static async updateUserPermissions(
    { http }: BaseClient,
    p: RequestData<{ userId: string }, UpdateUserPermissionViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/user/${p.params.userId}/permissions`, p.body);
  }
  static async updateUserRoles(
    { http }: BaseClient,
    p: RequestData<{ userId: string }, UpdateUserRolesViewModel>
  ): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/user/${p.params.userId}/roles`, p.body);
  }
  static async deleteUser({ http }: BaseClient, p: RequestData<{ id: string }, void>): Promise<AxiosResponse<void, any>> {
    return await http.post(`/admin/user/${p.params.id}`);
  }

  static async getInvites({ http }: BaseClient): Promise<AxiosResponse<InviteViewModel[], any>> {
    return await http.get("/admin/invite");
  }
  static async createInvite({ http }: BaseClient, p: RequestData<void, CreateInviteViewModel>): Promise<AxiosResponse<void, any>> {
    return await http.post("/admin/invite", p.body);
  }
  static async deleteInvite({ http }: BaseClient, p: RequestData<{ mail: string }, void>): Promise<AxiosResponse<void, any>> {
    return await http.delete(`/admin/invite/${p.params.mail}`);
  }
}
